@@ -1,7 +1,7 @@
-/* Copyright (c) 2019-2020 The Khronos Group Inc.
- * Copyright (c) 2019-2020 Valve Corporation
- * Copyright (c) 2019-2020 LunarG, Inc.
- * Copyright (C) 2019-2020 Google Inc.
+/* Copyright (c) 2019-2021 The Khronos Group Inc.
+ * Copyright (c) 2019-2021 Valve Corporation
+ * Copyright (c) 2019-2021 LunarG, Inc.
+ * Copyright (C) 2019-2021 Google Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -29,6 +29,7 @@
 #include <map>
 #include <utility>
 #include <cstdint>
+#include "vk_layer_data.h"
 
 #define RANGE_ASSERT(b) assert(b)
 
@@ -72,6 +73,8 @@ struct range {
         return *this;
     }
 
+    inline range operator+(const index_type &offset) const { return range(begin + offset, end + offset); }
+
     // for a reversible/transitive < operator compare first on begin and then end
     // only less or begin is less or if end is less when begin is equal
     bool operator<(const range &rhs) const {
@@ -139,9 +142,6 @@ class range_view {
     const Range &range_;
 };
 
-template <typename Container>
-using const_correct_iterator = decltype(std::declval<Container>().begin());
-
 // Type parameters for the range_map(s)
 struct insert_range_no_split_bounds {
     const static bool split_boundaries = false;
@@ -1401,11 +1401,12 @@ class cached_lower_bound_impl {
 
     inline iterator lower_bound(const index_type &index) { return map_->lower_bound(key_type(index, index + 1)); }
     inline bool at_end(const iterator &it) const { return it == end_; }
-    inline bool at_end() const { return at_end(lower_bound_); }
 
     bool is_lower_than(const index_type &index, const iterator &it) { return at_end(it) || (index < it->first.end); }
 
   public:
+    // The cached lower bound knows the parent map, and thus can tell us this...
+    inline bool at_end() const { return at_end(lower_bound_); }
     // includes(index) is a convenience function to test if the index would be in the currently cached lower bound
     bool includes(const index_type &index) const { return !at_end() && lower_bound_->first.includes(index); }
 
@@ -1474,6 +1475,12 @@ class cached_lower_bound_impl {
         return *this;
     }
 
+    // For times when the application knows what it's done to the underlying map... (with assert in set_value)
+    cached_lower_bound_impl &invalidate(const iterator &hint, index_type index) {
+        set_value(index, hint);
+        return *this;
+    }
+
     cached_lower_bound_impl &invalidate() { return invalidate(index_); }
 
     // Allow a hint for a *valid* lower bound for current index
@@ -1501,6 +1508,9 @@ class cached_lower_bound_impl {
         }
     }
 
+    Map &map() { return *map_; }
+    const Map &map() const { return *map_; }
+
     // Default constructed object reports valid (correctly) as false, but otherwise will fail (assert) under nearly any use.
     cached_lower_bound_impl()
         : map_(nullptr), end_(), pos_(index_, lower_bound_, valid_), index_(0), lower_bound_(), valid_(false) {}
@@ -1521,6 +1531,26 @@ const MappedType &evaluate(const CachedLowerBound &clb, const MappedType &defaul
     return default_value;
 }
 
+// Split a range into pieces bound by the interection of the interator's range and the supplied range
+template <typename Iterator, typename Map, typename Range>
+Iterator split(Iterator in, Map &map, const Range &range) {
+    assert(in != map.end());  // Not designed for use with invalid iterators...
+    const auto in_range = in->first;
+    const auto split_range = in_range & range;
+
+    if (split_range.empty()) return map.end();
+
+    auto pos = in;
+    if (split_range.begin != in_range.begin) {
+        pos = map.split(pos, split_range.begin, sparse_container::split_op_keep_both());
+        ++pos;
+    }
+    if (split_range.end != in_range.end) {
+        pos = map.split(pos, split_range.end, sparse_container::split_op_keep_both());
+    }
+    return pos;
+}
+
 // Parallel iterator
 // Traverse to range maps over the the same range, but without assumptions of aligned ranges.
 // ++ increments to the next point where on of the two maps changes range, giving a range over which the two
@@ -1620,12 +1650,21 @@ class parallel_iterator {
         seek(start);
         return *this;
     }
+
     parallel_iterator &invalidate_A() {
         const index_type index = range_.begin;
         pos_A_.invalidate(static_cast<index_type_A>(index));
         range_ = key_type(index, index + compute_delta());
         return *this;
     }
+
+    parallel_iterator &invalidate_A(const iterator_A &hint) {
+        const index_type index = range_.begin;
+        pos_A_.invalidate(hint, static_cast<index_type_A>(index));
+        range_ = key_type(index, index + compute_delta());
+        return *this;
+    }
+
     parallel_iterator &invalidate_B() {
         const index_type index = range_.begin;
         pos_B_.invalidate(static_cast<index_type_B>(index));
@@ -1633,20 +1672,36 @@ class parallel_iterator {
         return *this;
     }
 
+    parallel_iterator &invalidate_B(const iterator_B &hint) {
+        const index_type index = range_.begin;
+        pos_B_.invalidate(hint, static_cast<index_type_B>(index));
+        range_ = key_type(index, index + compute_delta());
+        return *this;
+    }
+
+    parallel_iterator &trim_A() {
+        if (pos_A_->valid && (range_ != pos_A_->lower_bound->first)) {
+            split(pos_A_->lower_bound, pos_A_.map(), range_);
+            invalidate_A();
+        }
+        return *this;
+    }
+
     // The return is const because we are sharing the internal state directly.
     const value_type &operator*() const { return pos_; }
     const value_type *operator->() const { return &pos_; }
 };
 
-template <typename RangeMap, typename SourceIterator = typename RangeMap::const_iterator>
-bool splice(RangeMap *to, const RangeMap &from, value_precedence arbiter, SourceIterator begin, SourceIterator end) {
+template <typename DstRangeMap, typename SrcRangeMap, typename Updater,
+          typename SourceIterator = typename SrcRangeMap::const_iterator>
+bool splice(DstRangeMap &to, const SrcRangeMap &from, SourceIterator begin, SourceIterator end, const Updater &updater) {
     if (from.empty() || (begin == end) || (begin == from.cend())) return false;  // nothing to merge.
 
-    using ParallelIterator = parallel_iterator<RangeMap, const RangeMap>;
-    using Key = typename RangeMap::key_type;
-    using CachedLowerBound = cached_lower_bound_impl<RangeMap>;
-    using ConstCachedLowerBound = cached_lower_bound_impl<const RangeMap>;
-    ParallelIterator par_it(*to, from, begin->first.begin);
+    using ParallelIterator = parallel_iterator<DstRangeMap, const SrcRangeMap>;
+    using Key = typename SrcRangeMap::key_type;
+    using CachedLowerBound = cached_lower_bound_impl<DstRangeMap>;
+    using ConstCachedLowerBound = cached_lower_bound_impl<const SrcRangeMap>;
+    ParallelIterator par_it(to, from, begin->first.begin);
     bool updated = false;
     while (par_it->range.non_empty() && par_it->pos_B->lower_bound != end) {
         const Key &range = par_it->range;
@@ -1658,33 +1713,69 @@ bool splice(RangeMap *to, const RangeMap &from, value_precedence arbiter, Source
             // Because of how the parallel iterator walk, "to" is valid over the whole range or it isn't (ranges don't span
             // transitions between map entries or between valid and invalid ranges)
             if (to_lb->valid) {
-                // Only rewrite this range if source is preferred (and the value differs)
-                // TODO determine if equality checks are always wanted. (for example heavyweight values)
-                if (arbiter == value_precedence::prefer_source && (write_it->second != read_it->second)) {
-                    // Both ranges occupied and source is preferred and from differs from to
-                    if (write_it->first == range) {
-                        // we're writing the whole destination range, so just set the value
-                        write_it->second = read_it->second;
-                    } else {
-                        to->overwrite_range(write_it, std::make_pair(range, read_it->second));
-                        par_it.invalidate_A();  // we've changed map 'to' behind to_lb's back... let it know.
-                    }
-                    updated = true;
+                if (write_it->first == range) {
+                    // if the source and destination ranges match we can overwrite everything
+                    updated |= updater.update(write_it->second, read_it->second);
+                } else {
+                    // otherwise we need to split the destination range.
+                    auto value_to_update = write_it->second; // intentional copy
+                    updated |= updater.update(value_to_update, read_it->second);
+                    auto intersected_range = write_it->first & range;
+                    to.overwrite_range(to_lb->lower_bound, std::make_pair(intersected_range, value_to_update));
+                    par_it.invalidate_A();  // we've changed map 'to' behind to_lb's back... let it know.
                 }
             } else {
                 // Insert into the gap.
-                to->insert(write_it, std::make_pair(range, read_it->second));
-                par_it.invalidate_A();  // we've changed map 'to' behind to_lb's back... let it know.
-                updated = true;
+                auto opt = updater.insert(read_it->second);
+                if (opt) {
+                    to.insert(write_it, std::make_pair(range, std::move(*opt)));
+                    updated = true;
+                    par_it.invalidate_A();  // we've changed map 'to' behind to_lb's back... let it know.
+                }
             }
         }
         ++par_it;  // next range over which both 'to' and 'from' stay constant
     }
     return updated;
 }
+// And short hand for "from begin to end"
+template <typename DstRangeMap, typename SrcRangeMap, typename Updater>
+bool splice(DstRangeMap &to, const SrcRangeMap &from, const Updater &updater) {
+    return splice(to, from, from.cbegin(), from.cend(), updater);
+}
+
+template <typename T>
+struct update_prefer_source {
+    bool update(T &dst, const T &src) const {
+        if (dst != src) {
+            dst = src;
+            return true;
+        }
+        return false;
+    }
+
+    layer_data::optional<T> insert(const T &src) const { return layer_data::optional<T>(layer_data::in_place, src); }
+};
+
+template <typename T>
+struct update_prefer_dest {
+    bool update(T &dst, const T &src) const { return false; }
+
+    layer_data::optional<T> insert(const T &src) const { return layer_data::optional<T>(layer_data::in_place, src); }
+};
+
+template <typename RangeMap, typename SourceIterator = typename RangeMap::const_iterator>
+bool splice(RangeMap &to, const RangeMap &from, value_precedence arbiter, SourceIterator begin, SourceIterator end) {
+    if (arbiter == value_precedence::prefer_source) {
+        return splice(to, from, from.cbegin(), from.cend(), update_prefer_source<typename RangeMap::mapped_type>());
+    } else {
+        return splice(to, from, from.cbegin(), from.cend(), update_prefer_dest<typename RangeMap::mapped_type>());
+    }
+}
+
 // And short hand for "from begin to end"
 template <typename RangeMap>
-bool splice(RangeMap *to, const RangeMap &from, value_precedence arbiter) {
+bool splice(RangeMap &to, const RangeMap &from, value_precedence arbiter) {
     return splice(to, from, arbiter, from.cbegin(), from.cend());
 }
 
