@@ -1,8 +1,8 @@
 #!/usr/bin/python3 -i
 #
-# Copyright (c) 2015-2017, 2019-2020 The Khronos Group Inc.
-# Copyright (c) 2015-2017, 2019-2020 Valve Corporation
-# Copyright (c) 2015-2017, 2019-2020 LunarG, Inc.
+# Copyright (c) 2015-2017, 2019-2021 The Khronos Group Inc.
+# Copyright (c) 2015-2017, 2019-2021 Valve Corporation
+# Copyright (c) 2015-2017, 2019-2021 LunarG, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,10 +25,10 @@ from collections import namedtuple, OrderedDict
 # Copyright text prefixing all headers (list of strings).
 prefixStrings = [
     '/*',
-    '** Copyright (c) 2015-2017, 2019 The Khronos Group Inc.',
-    '** Copyright (c) 2015-2017, 2019 Valve Corporation',
-    '** Copyright (c) 2015-2017, 2019 LunarG, Inc.',
-    '** Copyright (c) 2015-2017, 2019 Google Inc.',
+    '** Copyright (c) 2015-2017, 2019-2021 The Khronos Group Inc.',
+    '** Copyright (c) 2015-2017, 2019-2021 Valve Corporation',
+    '** Copyright (c) 2015-2017, 2019-2021 LunarG, Inc.',
+    '** Copyright (c) 2015-2017, 2019-2021 Google Inc.',
     '**',
     '** Licensed under the Apache License, Version 2.0 (the "License");',
     '** you may not use this file except in compliance with the License.',
@@ -61,6 +61,7 @@ platform_dict = {
     'xlib_xrandr' : 'VK_USE_PLATFORM_XLIB_XRANDR_EXT',
     'provisional' : 'VK_ENABLE_BETA_EXTENSIONS',
     'directfb' : 'VK_USE_PLATFORM_DIRECTFB_EXT',
+    'screen' : 'VK_USE_PLATFORM_SCREEN_QNX',
 }
 
 #
@@ -73,6 +74,13 @@ def GetFeatureProtect(interface):
         protect = platform_dict[platform]
     return protect
 
+# Return the _EXTENSION_NAME define for the given extension block
+def GetNameDefine(interface):
+    for enum in interface.findall('require/enum'):
+        if enum.get('name', '').endswith('_EXTENSION_NAME'):
+            return enum.get('name')
+    raise Exception(f'Could find name define for {extension.get("name")}')
+
 # Return a dict containing the dispatchable/non-dispatchable type of every handle
 def GetHandleTypes(tree):
     # Extend OrderedDict with common handle operations
@@ -137,6 +145,16 @@ def GetTypeCategories(tree):
             type_categories[name] = elem.get('category')
     return type_categories
 
+# Return a dict containing platform guard for every type
+def GetTypeGuards(tree):
+    type_guards = OrderedDict()
+    for ext_elem in tree.findall('extensions/extension'):
+        ext_guard = platform_dict.get(ext_elem.get('platform'))
+        if ext_guard:
+            for type_elem in ext_elem.findall('require/type'):
+                type_guards[type_elem.get('name')] = ext_guard
+    return type_guards
+
 # Treats outdents a multiline string by the leading whitespace on the first line
 # Optionally indenting by the given prefix
 def Outdent(string_in, indent=''):
@@ -152,6 +170,19 @@ def Outdent(string_in, indent=''):
     outdent = re.sub(fake_indent, indent, string_out)
     return outdent[1:]
 
+# Wrap a statement with the supplied guard, preserving the newline-ness of the input statement
+def Guarded(ifdef, value):
+    if ifdef is not None:
+        if value.endswith('\n'):
+            value, trailing_newline = value[:-1], value[-1:]
+        else:
+            trailing_newline = ''
+        return f'#ifdef {ifdef}\n' \
+               f'{value}\n' \
+               f'#endif{trailing_newline}'
+    else:
+        return value
+
 
 # helper to define paths relative to the repo root
 def repo_relative(path):
